from django.urls import path
from .views import SenderListView, SenderCreateView, SenderDetailRetrieveView, SenderDetailUpdateView

app_name = 'sender'

urlpatterns = [
    # sender api view
    path('', SenderListView.as_view(), name='senders-list'),
    path('', SenderCreateView.as_view(), name='senders-create'),
    path('<int:pk>', SenderDetailRetrieveView.as_view(), name='sender-entity-by-id-retrieve'),
    path('<int:pk>', SenderDetailUpdateView.as_view(), name='sender-entity-by-id-update'),
    # path('attr', SenderDetailUpdateView.as_view(), name='sender-list-attribute'),
]


from rest_framework import generics

from .models import Sender
from .serializers import SenderSerializer


# Create your views here.


class SenderListView(generics.ListAPIView):
    """
    the api for listing all senders entity
    """
    queryset = Sender.objects.all()
    serializer_class = SenderSerializer


class SenderCreateView(generics.CreateAPIView):
    """
    the api for creating one specific sender entity
    """
    queryset = Sender.objects.all()
    serializer_class = SenderSerializer


class SenderDetailRetrieveView(generics.RetrieveAPIView):
    """
    the api for showing the sender entity detail
    """
    queryset = Sender.objects.all()
    serializer_class = SenderSerializer


class SenderDetailUpdateView(generics.UpdateAPIView):
    """
    the api for updateing the sender entity detail
    """
    queryset = Sender.objects.all()
    serializer_class = SenderSerializer

# class SenderListWithAttr():



# Generated by Django 2.2.6 on 2019-12-21 08:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('beneficiary', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='beneficiary',
            old_name='data_of_birth',
            new_name='date_of_birth',
        ),
    ]



from rest_framework import serializers

from .models import Sender


class SenderSerializer(serializers.ModelSerializer):

    class Meta:
        model = Sender
        fields = ['id', 'lastname', 'lastname2', 'middlename', 'firstname', 'nativename',
                  'nationality_country_iso_code', 'code', 'date_of_birth', 'country_of_birth_iso_code',
                  'gender', 'address', 'postal_code', 'city', 'country_iso_code', 'msisdn', 'email',
                  'id_type', 'id_country_iso_code', 'id_number', 'id_delivery_date', 'id_expiration_date', 'occupation',
                  'bank', 'bank_account', 'card', 'province_state', 'beneficiary_relationship', 'source_of_funds']



"""thunes URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path, include
from rest_framework.documentation import include_docs_urls

adminpatterns = [
    path('admin/', admin.site.urls),
]

documentspatterns = [
    path('api/docs/', include_docs_urls(title='Thunes Test Apis')),
]

urlpatterns = adminpatterns + documentspatterns + [
    path('api/v1/', include([
        path('senders/', include('sender.urls', namespace='sender_api')),
        path('beneficiaries/', include('beneficiary.urls', namespace='beneficiary_api')),
        path('transactions/', include('transaction.urls', namespace='transaction_api')),
        path('quotation/', include('quotation.urls', namespace='quotation_api')),
    ])),
]



# Generated by Django 2.2.6 on 2019-12-23 08:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('transaction', '0007_transactionsenderbeneficiary'),
    ]

    operations = [
        migrations.DeleteModel(
            name='TransactionSenderBeneficiary',
        ),
    ]



from django.urls import path

from .views import BeneficiaryListView, BeneficiaryCreateView, BeneficiaryDetailRetrieveView, \
    BeneficiaryDetailUpdateView

app_name = 'beneficiary'

urlpatterns = [
    # beneficiary api view
    path('', BeneficiaryListView.as_view(), name='beneficiaries-list'),
    path('', BeneficiaryCreateView.as_view(), name='beneficiaries-create'),
    path('<int:pk>', BeneficiaryDetailRetrieveView.as_view(), name='beneficiary-entity-by-id-retrieve'),
    path('<int:pk>', BeneficiaryDetailUpdateView.as_view(), name='beneficiary-entity-by-id-update'),
    # path('attr'),
]


from django.urls import path
from .views import TransactionListView, TransactionCreateView, TransactionDetailRetrieveView, \
    BeneficiaryListBySenderName, TransactionConfirmView

app_name = 'transaction'

urlpatterns = [
    # beneficiary api view
    path('<int:quotation_id>', TransactionCreateView.as_view(), name='transaction-create'),
    path('', TransactionListView.as_view(), name='transaction-list'),
    path('confirm/<int:transaction_id>', TransactionConfirmView.as_view(), name='transaction-entity-by-id-confirm'),
    path('detail/<int:pk>', TransactionDetailRetrieveView.as_view(), name='transaction-entity-by-id-retrieve'),
    path('sender', BeneficiaryListBySenderName.as_view(), name='transaction-test')
    # path('attr'),
]


import json
import random

import requests
from rest_framework.response import Response
from rest_framework.views import APIView
from thunes.APIAccessGrant import APIAccessGrant

from .models import Quotation, Source, Destination


# Create your views here.


class QuotationDetailView(APIView):
    """
    the api for showing the quotation entity detail by ID
    """
    result_body = {}

    def get(self, request, id, format=None):
        APIObject = APIAccessGrant('/quotations/' + str(id))
        apiprams = APIObject.parmReturn()

        response = requests.get(apiprams['urlparm'], headers={
            'X-TransferTo-apikey': apiprams['apikey'],
            'X-TransferTo-nonce': apiprams['nonce'],
            'X-TransferTo-hmac': apiprams['hmac'],
            'Date': apiprams['date'],
            # 'Content-Type': 'application/json',
        })
        # response_json = response.json()
        # self.result_body['id'] = response_json['id']
        # self.result_body['external_id'] = response_json['external_id']
        # self.result_body['mode'] = response_json['mode']
        # self.result_body['destination'] = response_json['destination']
        # self.result_body['source'] = response_json['source']
        return Response(response.json())


class QuotationCreateView(APIView):
    """
    the api for creating the quotation entity
    """
    quotation_body = {
        "external_id": "148118432398",
        "payer_id": "1669",
        "mode": "SOURCE_AMOUNT",
        "source": {
            "amount": 10,
            "country_iso_code": "SGP",
            "currency": "SGD"
        },
        "destination": {
            "amount": None,
            "currency": "GHS"
        }
    }

    def post(self, request, *args, **kwargs):
        # import random to generate different external id
        self.quotation_body['external_id'] = str(148118432398 + random.randint(0, 10000000000))
        APIObject = APIAccessGrant('/quotations/')
        apiprams = APIObject.parmReturn()

        # the method to get the key
        (response_body, value), = request.data.items()

        self.quotation_body['source'] = json.loads(response_body)['source']
        self.quotation_body['destination'] = json.loads(response_body)['destination']

        # Enable this function to store the quotation data
        # self.createquotationentity(request.data)

        response = requests.post(apiprams['urlparm'], headers={
            'X-TransferTo-apikey': apiprams['apikey'],
            'X-TransferTo-nonce': apiprams['nonce'],
            'X-TransferTo-hmac': apiprams['hmac'],
            'Date': apiprams['date'],
            'Content-Type': 'application/json',
        }, json=self.quotation_body)
        return Response(response.json())

    def createquotationentity(self, data):
        try:
            quotationentity = Quotation.objects.get(external_id=data['external_id'])
        except Quotation.DoesNotExist:
            sourceentity = Source.objects.create(
                country_iso_code=data['source']['country_iso_code'],
                currency=data['source']['currency'],
                amount=data['source']['amount']
            )
            destinationentity = Destination.objects.create(
                currency=data['destination']['currency'],
                amount=data['destination']['amount']
            )
            quotationentity = Quotation.objects.create(
                external_id=data['external_id'],
                payer_id=data['payer_id'],
                mode=data['mode'],
                source=sourceentity,
                destination=destinationentity
            )


class ServerPing(APIView):

    def get(self, request, format=None):
        """
        the function to test the backend API's connectivity
        """
        APIObject = APIAccessGrant('')
        apiprams = APIObject.parmReturn()

        pingUrl = apiprams['urlparm'].replace('v1/money-transfer', 'ping')

        # Enable this function to store the quotation data
        # self.createquotationentity(request.data)

        response = requests.get(pingUrl, headers={
            'X-TransferTo-apikey': apiprams['apikey'],
            'X-TransferTo-nonce': apiprams['nonce'],
            'X-TransferTo-hmac': apiprams['hmac'],
            'Date': apiprams['date'],
        })
        return Response(response.json())




# Generated by Django 2.2.6 on 2019-12-21 05:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('quotation', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='destination',
            name='amount',
            field=models.IntegerField(blank=True, max_length=10, verbose_name='Source amount'),
        ),
        migrations.AlterField(
            model_name='source',
            name='amount',
            field=models.IntegerField(blank=True, verbose_name='Source amount'),
        ),
    ]



from django.urls import path

from .views import QuotationCreateView, QuotationDetailView, ServerPing

app_name = 'quotation'

urlpatterns = [
    # sender api view
    path('ping', ServerPing.as_view(), name='Server-Ping'),
    path('create', QuotationCreateView.as_view(), name='quotation-create'),
    path('<int:id>', QuotationDetailView.as_view(), name='quotation-detail'),
]


# Generated by Django 2.2.6 on 2019-12-21 08:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('transaction', '0003_auto_20191221_1643'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='creditpartyidentity',
            options={'verbose_name': 'Credit Party Identity'},
        ),
        migrations.AlterField(
            model_name='transaction',
            name='creation_date',
            field=models.DateTimeField(auto_now=True, max_length=50, verbose_name='Creation date in HTTP format'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='retail_rate',
            field=models.DecimalField(blank=True, decimal_places=15, max_digits=15, verbose_name='Retail rate'),
        ),
    ]



from .base import *

# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

# Allowed Hosts
ALLOWED_HOSTS = ['*']

# the database setting for sqlite database
# DATABASES = {
#     'default': {
#         'ENGINE': 'django.db.backends.sqlite3',
#         'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
#     }
# }

# the database setting for postgres
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'thunes',
        'USER': 'postgres',
        'PASSWORD': 'postgres',
        'HOST': 'postgres',
        'PORT': 5432,
        # 'CONN_MAX_AGE': 5 * 60,
        # 'OPTIONS': {'charset': 'utf8mb4'}
    }
}

# CORS Settings
CORS_ALLOW_CREDENTIALS = True
CORS_ORIGIN_ALLOW_ALL = True
CORS_ORIGIN_WHITELIST = (
    'http://localhost:3000',
    'http://localhost:8000',
    'http://localhost:8080',
)
CORS_ALLOW_METHODS = (
    'DELETE',
    'GET',
    'OPTIONS',
    'PATCH',
    'POST',
    'PUT',
    'VIEW',
)
CORS_ALLOW_HEADERS = (
    'XMLHttpRequest',
    'X_FILENAME',
    'accept-encoding',
    'authorization',
    'content-type',
    'dnt',
    'origin',
    'user-agent',
    'x-csrftoken',
    'x-requested-with',
    'Pragma',
)



from django.db import models

from sender.models import Sender
from beneficiary.models import Beneficiary


# Create your models here.
class Transaction(models.Model):
    TRANSACTION_STATUS = (
        ('10000', 'CREATED'),
        ('20000', 'CONFIRMED'),
        ('20110', 'CONFIRMED-UNDER-REVIEW-SLS'),
        ('20150', 'CONFIRMED-WAITING-FOR-PICKUP'),
        ('30000', 'REJECTED'),
        ('30110', 'REJECTED-SLS-SENDER'),
        ('30120', 'REJECTED-SLS-BENEFICIARY'),
        ('30200', 'REJECTED-INVALID-BENEFICIARY'),
        ('30201', 'REJECTED-BARRED-BENEFICIARY'),
        ('30210', 'REJECTED-INVALID-BENEFICIARY-DETAILS'),
        ('30305', 'REJECTED-LIMITATIONS-ON-TRANSACTION-VALUE'),
        ('30310', 'REJECTED-LIMITATIONS-ON-SENDER-VALUE'),
        ('30320', 'REJECTED-LIMITATIONS-ON-BENEFICIARY-VALUE'),
        ('30330', 'REJECTED-LIMITATIONS-ON-ACCOUNT-VALUE'),
        ('30350', 'REJECTED-LIMITATIONS-ON-SENDER-QUANTITY'),
        ('30360', 'REJECTED-LIMITATIONS-ON-BENEFICIARY-QUANTITY'),
        ('30370', 'REJECTED-LIMITATIONS-ON-ACCOUNT-QUANTITY'),
        ('30400', 'REJECTED-PAYER-CURRENTLY-UNAVAILABLE'),
        ('30500', 'REJECTED-INSUFFICIENT-BALANCE'),
        ('40000', 'CANCELLED'),
        ('50000', 'SUBMITTED'),
        ('60000', 'AVAILABLE'),
        ('70000', 'COMPLETED'),
        ('80000', 'REVERSED'),
        ('90000', 'DECLINED'),
        ('90110', 'DECLINED-SLS-SENDER'),
        ('90120', 'DECLINED-SLS-BENEFICIARY'),
        ('90200', 'DECLINED-INVALID-BENEFICIARY'),
        ('90201', 'DECLINED-BARRED-BENEFICIARY'),
        ('90202', 'DECLINED-UNSUPPORTED-BENEFICIARY'),
        ('90210', 'DECLINED-INVALID-BENEFICIARY-DETAILS'),
        ('90305', 'DECLINED-LIMITATIONS-ON-TRANSACTION-VALUE'),
        ('90310', 'DECLINED-LIMITATIONS-ON-SENDER-VALUE'),
        ('90320', 'DECLINED-LIMITATIONS-ON-BENEFICIARY-VALUE'),
        ('90330', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE'),
        ('90331', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-DAILY'),
        ('90332', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-WEEKLY'),
        ('90333', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-MONTHLY'),
        ('90334', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-YEARLY'),
        ('90350', 'DECLINED-LIMITATIONS-ON-SENDER-QUANTITY'),
        ('90360', 'DECLINED-LIMITATIONS-ON-BENEFICIARY-QUANTITY'),
        ('90370', 'DECLINED-LIMITATIONS-ON-ACCOUNT-QUANTITY'),
        ('90400', 'DECLINED-PAYER-CURRENTLY-UNAVAILABLE')
    )
    TRANSACTION_STATUS_CLASS = (
        ('1', 'CREATED'),
        ('2', 'CONFIRMED'),
        ('3', 'REJECTED'),
        ('4', 'CANCELLED'),
        ('5', 'SUBMITTED'),
        ('6', 'AVAILABLE'),
        ('7', 'COMPLETED'),
        ('8', 'REVERSED'),
        ('9', 'DECLINED'),
    )
    status = models.CharField(max_length=50, blank=False,
                              default='10000', choices=TRANSACTION_STATUS, verbose_name='Transaction status code')
    status_message = models.CharField(max_length=50, blank=False, default='10000', choices=TRANSACTION_STATUS,
                                      verbose_name='Transaction status description')
    status_class = models.CharField(max_length=50, blank=False, default='1', choices=TRANSACTION_STATUS_CLASS,
                                    verbose_name='Transaction status class')
    status_class_message = models.CharField(max_length=50, blank=False, default='1', choices=TRANSACTION_STATUS_CLASS)
    external_id = models.CharField(max_length=50, blank=False, verbose_name='External ID')
    external_code = models.CharField(max_length=50, blank=False, verbose_name='External reference code')
    payer_transaction_reference = models.CharField(max_length=50, blank=True,
                                                   verbose_name='Payer transaction reference')
    payer_transaction_code = models.CharField(max_length=50, blank=True, verbose_name='Payer transaction code')
    creation_date = models.DateTimeField(max_length=50, auto_now=True, verbose_name='Creation date in HTTP format')
    expiration_date = models.DateTimeField(max_length=50, blank=True, null=True,
                                           verbose_name='Expiration date in HTTP format')
    sender = models.ForeignKey(Sender, related_name='senders', on_delete=models.CASCADE,
                               verbose_name='Sender information')
    beneficiary = models.ForeignKey(Beneficiary, related_name='beneficiaries',
                                    on_delete=models.CASCADE, verbose_name='Beneficiary information')
    callback_url = models.CharField(max_length=50, blank=False, verbose_name='Callback URL')
    wholesale_fx_rate = models.DecimalField(max_digits=15, blank=True, null=True, decimal_places=15,
                                            verbose_name='Wholesale FX rate')
    retail_rate = models.DecimalField(max_digits=15, blank=True, null=True, decimal_places=15,
                                      verbose_name='Retail rate')
    retail_fee = models.DecimalField(max_digits=15, blank=True, null=True, decimal_places=15, verbose_name='Retail fee')
    retail_fee_currency = models.CharField(max_length=50, blank=False,
                                           verbose_name='Retail fee currency in ISO 4217 format')
    purpose_of_remittance = models.CharField(max_length=50, blank=False, verbose_name='Purpose of remittance')
    additional_information_1 = models.CharField(max_length=50, blank=True, verbose_name='Additional information')
    additional_information_2 = models.CharField(max_length=50, blank=True, verbose_name='Additional information')
    additional_information_3 = models.CharField(max_length=50, blank=True, verbose_name='Additional information')

    def __str__(self):
        return 'Transaction {}'.format(self.external_id)

    class Meta:
        index_together = (('sender', 'beneficiary'),)
        verbose_name = 'transaction'
        db_table = 'transaction'


class CreditPartyIdentity(models.Model):
    msisdn = models.CharField(max_length=50, blank=False, verbose_name='MSISDN in international format')
    bank_account_number = models.CharField(max_length=50, blank=False, verbose_name='Bank account number')
    swift_bic_code = models.CharField(max_length=50, blank=False, verbose_name='SWIFT-BIC code')

    def __str__(self):
        return "Credit Party Identity {} {} {}".format(self.msisdn, self.bank_account_number, self.swift_bic_code)

    class Meta:
        db_table = 'Credit Party Identity'
        verbose_name = 'Credit Party Identity'


# class TransactionSenderBeneficiary(models.Model):
#     sender = models.OneToOneField(Sender, verbose_name='sender entity in this transaction', on_delete=models.CASCADE)
#     beneficiary = models.OneToOneField(Beneficiary, verbose_name='beneficiary entity in this transaction',
#                                        on_delete=models.CASCADE)




from rest_framework import serializers

from .models import Transaction
# from sender.serializers import SenderSerializer
# from beneficiary.serializers import BeneficiarySerializer


class TransactionSerializer(serializers.ModelSerializer):
    # senders_entity = SenderSerializer()
    # beneficiaries_entities = BeneficiarySerializer()

    class Meta:
        model = Transaction
        fields = ['status', 'status_message', 'status_class', 'status_class_message', 'external_id',
                  'external_code', 'payer_transaction_reference', 'payer_transaction_code', 'creation_date',
                  'expiration_date', 'callback_url', 'wholesale_fx_rate', 'retail_rate',
                  'retail_fee', 'retail_fee_currency', 'purpose_of_remittance']



# Generated by Django 2.2.6 on 2019-12-21 08:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('transaction', '0004_auto_20191221_1656'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transaction',
            name='retail_rate',
            field=models.DecimalField(blank=True, decimal_places=15, max_digits=15, null=True, verbose_name='Retail rate'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='wholesale_fx_rate',
            field=models.DecimalField(blank=True, decimal_places=15, max_digits=15, null=True, verbose_name='Wholesale FX rate'),
        ),
    ]



from django.contrib import admin

from .models import Quotation, Source, Destination


# Register your models here.


@admin.register(Quotation)
class QuotationAdmin(admin.ModelAdmin):
    list_display = ['external_id', 'payer_id', 'mode', 'source', 'destination']


@admin.register(Source)
class SourceAdmin(admin.ModelAdmin):
    list_display = ['country_iso_code', 'currency', 'amount']


@admin.register(Destination)
class DestinationAdmin(admin.ModelAdmin):
    list_display = ['currency', 'amount']




from rest_framework import generics

from .models import Beneficiary
from .serializers import BeneficiarySerializer

# Create your views here.


class BeneficiaryListView(generics.ListAPIView):
    """
    the api for listing all beneficiaries entity
    """
    queryset = Beneficiary.objects.all()
    serializer_class = BeneficiarySerializer


class BeneficiaryCreateView(generics.CreateAPIView):
    """
    the api for creating one specific beneficiary entity
    """
    queryset = Beneficiary.objects.all()
    serializer_class = BeneficiarySerializer


class BeneficiaryDetailRetrieveView(generics.RetrieveAPIView):
    """
    the api for showing the one specific beneficiary entity by ID
    """
    queryset = Beneficiary.objects.all()
    serializer_class = BeneficiarySerializer


class BeneficiaryDetailUpdateView(generics.UpdateAPIView):
    """
    the api for updateing the beneficiary entity detail by ID
    """
    queryset = Beneficiary.objects.all()
    serializer_class = BeneficiarySerializer



from rest_framework import serializers

from .models import Beneficiary


class BeneficiarySerializer(serializers.ModelSerializer):

    class Meta:
        model = Beneficiary
        fields = '__all__'



import base64
import email
import hashlib
import time

from django.conf import settings
import sys
import os


class APIAccessGrant:

    def __init__(self, urlparm):
        """
        the function for initializing all required params.
        """
        self.apikey = settings.API_KEY
        self.apisecret = settings.API_SECRET
        self.url = settings.API_BASE_URL + urlparm

        self.nonce = int(time.time())
        self.date = email.utils.formatdate(usegmt=True)
        self.nonce_bytes = bytes(str(self.nonce).encode('utf-8'))

    def parmReturn(self):
        """
        the function for calculating all secret information and return a map entity.
        """
        secret = bytes(self.apisecret.encode('utf-8'))
        message = bytes((self.apikey + str(self.nonce) + self.date).encode('utf-8'))
        import hmac
        hmac = base64.b64encode(hmac.new(secret, message, digestmod=hashlib.sha256).digest())
        resultReturn = {'apikey': self.apikey, 'nonce': self.nonce_bytes,
                        'hmac': hmac, 'date': self.date, 'urlparm': self.url}
        return resultReturn



# Generated by Django 2.2.6 on 2019-12-21 05:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Destination',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('currency', models.CharField(max_length=10, verbose_name='Source currency in ISO 4217 format')),
                ('amount', models.IntegerField(max_length=10, verbose_name='Source amount')),
            ],
        ),
        migrations.CreateModel(
            name='Source',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('country_iso_code', models.CharField(max_length=10, verbose_name='Country code in ISO 3166-1 alpha-3 format')),
                ('currency', models.CharField(max_length=10, verbose_name='Source currency in ISO 4217 format')),
                ('amount', models.IntegerField(max_length=10, verbose_name='Source amount')),
            ],
        ),
        migrations.CreateModel(
            name='Quotation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('external_id', models.CharField(max_length=50, verbose_name='External reference ID')),
                ('payer_id', models.CharField(max_length=50, verbose_name='Payer ID')),
                ('mode', models.CharField(choices=[('SOURCE_AMOUNT', 'Quotation created by specifying desired source'), ('DESTINATION_AMOUNT', 'Quotation created by specifying desired destination')], default='SOURCE_AMOUNT', max_length=50, verbose_name='Quotation mode')),
                ('destination', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='destinationEntity', to='quotation.Destination', verbose_name='Destination information')),
                ('source', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='sourceEntity', to='quotation.Source', verbose_name='Source information')),
            ],
            options={
                'verbose_name': 'quotation',
                'db_table': 'quotation',
            },
        ),
    ]



from django.db import models

# Create your models here.


class Quotation(models.Model):
    QUOTATION_MODE = (
        ('SOURCE_AMOUNT', 'Quotation created by specifying desired source'),
        ('DESTINATION_AMOUNT', 'Quotation created by specifying desired destination'),
    )
    external_id = models.CharField(max_length=50, blank=False, verbose_name='External reference ID')
    payer_id = models.CharField(max_length=50, blank=False, verbose_name='Payer ID')
    mode = models.CharField(max_length=50, default='SOURCE_AMOUNT', choices=QUOTATION_MODE,
                            blank=False, verbose_name='Quotation mode')
    source = models.OneToOneField('Source', related_name='sourceEntity', on_delete=models.CASCADE,
                                  blank=False, verbose_name='Source information')
    destination = models.OneToOneField('Destination', related_name='destinationEntity', blank=False,
                                       on_delete=models.CASCADE, verbose_name='Destination information')

    class Meta:
        db_table = 'quotation'
        verbose_name = 'quotation'


class Source(models.Model):
    country_iso_code = models.CharField(max_length=10, blank=False,
                                        verbose_name='Country code in ISO 3166-1 alpha-3 format')
    currency = models.CharField(max_length=10, blank=False, verbose_name='Source currency in ISO 4217 format')
    amount = models.IntegerField(blank=True, verbose_name='Source amount')

    def __str__(self):
        return 'Source {}|{}|{}'.format(self.country_iso_code, self.currency, self.amount)


class Destination(models.Model):
    currency = models.CharField(max_length=10, blank=False, verbose_name='Source currency in ISO 4217 format')
    amount = models.IntegerField(blank=True, null=True, verbose_name='Source amount')

    def __str__(self):
        return 'Destination {}|{}'.format(self.currency, self.amount)




from django.contrib import admin
from .models import Sender

# Register your models here.
@admin.register(Sender)
class SenderAdmin(admin.ModelAdmin):
    list_display = ['lastname', 'firstname', 'nationality_country_iso_code', 'date_of_birth', 'gender',
                    'country_of_birth_iso_code', 'address', 'postal_code', 'city', 'country_iso_code',
                    'msisdn', 'email', 'id_type', 'id_country_iso_code', 'id_number', 'id_delivery_date',
                    'occupation', 'beneficiary_relationship']
    list_editable = ['date_of_birth', 'gender', 'id_type', 'id_delivery_date', 'beneficiary_relationship']



import requests
from rest_framework import generics
from rest_framework.response import Response
from rest_framework.views import APIView
from thunes.APIAccessGrant import APIAccessGrant


from django.db.models import Q

from .models import Transaction, CreditPartyIdentity
from .serializers import TransactionSerializer

from sender.models import Sender
from beneficiary.models import Beneficiary


# Create your views here.


class TransactionListView(generics.ListAPIView):
    """
    the api for listing all transactions entities
    """
    queryset = Transaction.objects.all()
    serializer_class = TransactionSerializer


class TransactionCreateView(APIView):
    """
    the api for creating an transaction entity based on the quotation entity
    """

    def post(self, request, quotation_id, *args, **kwargs):
        APIObject = APIAccessGrant('/quotations/' + str(quotation_id) + '/transactions')
        apiprams = APIObject.parmReturn()

        # print(request.data)
        self.transactioncreate(request.data)

        response = requests.post(apiprams['urlparm'], headers={
            'X-TransferTo-apikey': apiprams['apikey'],
            'X-TransferTo-nonce': apiprams['nonce'],
            'X-TransferTo-hmac': apiprams['hmac'],
            'Date': apiprams['date'],
            'Content-Type': 'application/json',
        }, json=request.data)
        return Response(response.json())

    def transactioncreate(self, data):
        """
        the function for creating the transaction entity
        """
        try:
            transactionentity = Transaction.objects.get(external_id=data['external_id'])
        except Transaction.DoesNotExist:
            senderentity = self.sendercreate(data['sender'])
            beneficiaryentity = self.beneficiarycreate(data['beneficiary'])
            self.creditpartyidentity(data['credit_party_identifier'])
            transactionentity = Transaction.objects.create(
                sender=senderentity,
                beneficiary=beneficiaryentity,
                external_id=data['external_id'],
                retail_fee_currency=data['retail_fee_currency'],
                purpose_of_remittance=data['purpose_of_remittance'],
                # retail_fee=data['retail_fee']
            )

    def sendercreate(self, senderdata):
        """
        the function for creating the sender entity
        """
        try:
            senderentity = Sender.objects.get(lastname=senderdata['lastname'], firstname=senderdata['firstname'])
            return senderentity
        except Sender.DoesNotExist:
            senderentity = Sender.objects.create(
                lastname=senderdata['lastname'],
                firstname=senderdata['firstname'],
                nationality_country_iso_code=senderdata['nationality_country_iso_code'],
                date_of_birth=senderdata['date_of_birth'],
                country_of_birth_iso_code=senderdata['country_of_birth_iso_code'],
                gender=senderdata['gender'],
                address=senderdata['address'],
                postal_code=senderdata['postal_code'],
                city=senderdata['city'],
                country_iso_code=senderdata['country_iso_code'],
                msisdn=senderdata['msisdn'],
                email=senderdata['email'],
                id_type=senderdata['id_type'],
                id_number=senderdata['id_number'],
                id_delivery_date=senderdata['id_delivery_date'],
                occupation=senderdata['occupation']
            )
        return senderentity

    def beneficiarycreate(self, beneficiarydata):
        """
        the function for creating the beneficiary entity
        """
        try:
            beneficiaryentity = Beneficiary.objects.get(lastname=beneficiarydata['lastname'],
                                                        firstname=beneficiarydata['firstname'])
            return beneficiaryentity
        except Beneficiary.DoesNotExist:
            beneficiaryentity = Beneficiary.objects.create(
                lastname=beneficiarydata['lastname'],
                firstname=beneficiarydata['firstname'],
                nationality_country_iso_code=beneficiarydata['nationality_country_iso_code'],
                date_of_birth=beneficiarydata['date_of_birth'],
                country_of_birth_iso_code=beneficiarydata['country_of_birth_iso_code'],
                gender=beneficiarydata['gender'],
                address=beneficiarydata['address'],
                postal_code=beneficiarydata['postal_code'],
                city=beneficiarydata['city'],
                country_iso_code=beneficiarydata['country_iso_code'],
                msisdn=beneficiarydata['msisdn'],
                email=beneficiarydata['email'],
                id_type=beneficiarydata['id_type'],
                id_number=beneficiarydata['id_number'],
                occupation=beneficiarydata['occupation']
            )
        return beneficiaryentity

    def creditpartyidentity(self, creditdata):
        """
         the function for creating the credit party identity entity
        """
        try:
            creditentity = CreditPartyIdentity.objects.get(
                msisdn=creditdata['msisdn'],
                bank_account_number=creditdata['bank_account_number'],
                swift_bic_code=creditdata['swift_bic_code']
            )
        except CreditPartyIdentity.DoesNotExist:
            creditentity = CreditPartyIdentity.objects.create(
                msisdn=creditdata['msisdn'],
                bank_account_number=creditdata['bank_account_number'],
                swift_bic_code=creditdata['swift_bic_code']
            )


class TransactionConfirmView(APIView):
    """
    the api for confirming the transaction has been created.
    """

    def post(self, request, transaction_id, *args, **kwargs):
        APIObject = APIAccessGrant('/transactions/' + str(transaction_id) + '/confirm')
        apiprams = APIObject.parmReturn()

        # print(request.data)
        # self.transactioncreate(request.data)
        # self.transactioncreate(request.data)

        response = requests.post(apiprams['urlparm'], headers={
            'X-TransferTo-apikey': apiprams['apikey'],
            'X-TransferTo-nonce': apiprams['nonce'],
            'X-TransferTo-hmac': apiprams['hmac'],
            'Date': apiprams['date'],
            'Content-Type': 'application/json',
        }, json=request.data)
        return Response(response.json())


class TransactionDetailRetrieveView(generics.RetrieveAPIView):
    """
    the api for showing the transactoin entity detail
    """
    queryset = Transaction.objects.all()
    serializer_class = TransactionSerializer


class TransactionDetailUpdateView(generics.UpdateAPIView):
    """
    the api for updating the transaction entity detail
    """
    queryset = Transaction.objects.all()
    serializer_class = TransactionSerializer


class BeneficiaryListBySenderName(generics.RetrieveAPIView):
    """
    the api for showing Beneficiary based on the given sender name
    """
    beneficiary_list = {}
    beneficiary_list_result = []

    def get(self, request, *args, **kwargs):
        lastname = self.request.GET.get('lastname', None)
        firstname = self.request.GET.get('firstname', None)
        # Add the clear function to not create too much memory space for these two para
        self.beneficiary_list.clear()
        self.beneficiary_list_result.clear()

        if lastname is None and firstname is None:
            pass
        elif lastname is None or firstname is None:
            pass
        else:
            senderentity = self.getsenderentity(lastname, firstname)
            beneficiaryentities = Transaction.objects.filter(Q(sender=senderentity)).values('beneficiary_id')
            # for loop to get a beneficiary map with beneficiary_id as key and
            # occurancy of this beneficiary as the value
            for beneficiary in beneficiaryentities:
                if beneficiary['beneficiary_id'] in self.beneficiary_list.keys():
                    self.beneficiary_list[beneficiary['beneficiary_id']] += 1
                else:
                    self.beneficiary_list[beneficiary['beneficiary_id']] = 1
            sorted(self.beneficiary_list.values())
            for key in self.beneficiary_list.keys():
                beneficiary_single_entity = self.getbeneficiary(key)
                self.beneficiary_list_result.append({'beneficiary_id': key,
                                                'beneficiary_name': beneficiary_single_entity.__str__(),
                                                'counter': self.beneficiary_list[key]})
        return Response(self.beneficiary_list_result)

    def getsenderentity(self, lastname, firstname):
        """
        the function for get the sender entity by its lastname and firstname
        """
        if firstname is not None and lastname is not None:
            try:
                return Sender.objects.get(firstname=firstname, lastname=lastname)
            except Sender.DoesNotExist:
                return None
        elif firstname is not None:
            try:
                return Sender.objects.get(firstname=firstname)
            except Sender.DoesNotExist:
                return None
        else:
            try:
                return Sender.objects.get(lastname=lastname)
            except Sender.DoesNotExist:
                return None
        return None

    def getbeneficiary(self, pk):
        try:
            return Beneficiary.objects.get(pk=pk)
        except Beneficiary.DoesNotExist:
            return None




from django.db import models


# Create your models here.
class Beneficiary(models.Model):
    GENDER = (
        ('MALE', 'Male'),
        ('FEMALE', 'Female'),
    )
    ID_TYPE = (
        ('PASSPORT', 'Passport'),
        ('NATIONAL_ID', 'National Identification Card'),
        ('DRIVING_LICENSE', 'Driving License'),
        ('SOCIAL_SECURITY', 'Social Security Card/Number'),
        ('TAX_ID', 'Tax Payer Identification Card/Number'),
        ('SENIOR_CITIZEN_ID', 'Senior Citizen Identification Card'),
        ('BIRTH_CERTIFICATE', 'Birth Certificate'),
        ('VILLAGE_ELDER_ID', 'Village Elder Identification Card'),
        ('RESIDENT_CARD', 'Permanent Residency Identification Card'),
        ('ALIEN_REGISTRATION', 'Alien Registration Certificate/Card'),
        ('PAN_CARD', 'PAN Card'),
        ('VOTERS_ID', 'Voter’s Identification Card'),
        ('HEALTH_CARD', 'Health Insurance Card/Number'),
        ('EMPLOYER_ID', 'Employer Identification Card'),
        ('OTHER', 'Others not listed')
    )
    lastname = models.CharField(max_length=50, blank=False, verbose_name='Last Name')
    lastname2 = models.CharField(max_length=50, blank=True, verbose_name='Additional last name(s)')
    middlename = models.CharField(max_length=50, blank=True, verbose_name='Middle Name')
    firstname = models.CharField(max_length=50, blank=False, verbose_name='First Name')
    nativename = models.CharField(max_length=50, blank=True, verbose_name='Full name in native characters')
    nationality_country_iso_code = models.CharField(max_length=10, blank=False,
                                                    verbose_name='National Country ISO Code')
    code = models.CharField(max_length=10, blank=True, verbose_name='code')
    date_of_birth = models.DateField(blank=False, verbose_name='Date Of Birth')
    country_of_birth_iso_code = models.CharField(max_length=10, blank=False, verbose_name='Country Of Birth ISO Code')
    gender = models.CharField(default='MALE', max_length=10, choices=GENDER, verbose_name='gender')
    address = models.CharField(max_length=50, blank=False, verbose_name='Address')
    postal_code = models.CharField(max_length=50, blank=False, verbose_name='Postal Code')
    city = models.CharField(max_length=50, blank=False, verbose_name='City')
    country_iso_code = models.CharField(max_length=50, blank=False, verbose_name='Country ISO Code')
    msisdn = models.CharField(max_length=50, blank=False, verbose_name='MSISDN in international format')
    email = models.EmailField(max_length=50, blank=False, verbose_name='Email Address')
    id_type = models.CharField(default='', choices=ID_TYPE, max_length=50, blank=False,
                               verbose_name='Presented identification type')
    id_country_iso_code = models.CharField(max_length=50, blank=False,
                                           verbose_name='ID country in ISO 3166-1 alpha-3 format')
    id_number = models.CharField(max_length=50, blank=False, verbose_name='Presented identification number')
    id_delivery_date = models.DateField(max_length=50, blank=True, null=True,
                                        verbose_name='ID delivery date in ISO 8601 format')
    id_expiration_date = models.DateField(max_length=50, blank=True, null=True,
                                          verbose_name='ID expiration date in ISO 8601 format')
    occupation = models.CharField(max_length=50, blank=False, verbose_name='Occupation')
    bank_accout_holder_name = models.CharField(max_length=50, blank=True, verbose_name='Bank account holder name')
    province_state = models.CharField(max_length=50, blank=True, verbose_name='Address province/state')

    def __str__(self):
        return 'Beneficiary Name: {} {}'.format(self.lastname, self.firstname)

    class Meta:
        verbose_name = 'beneficiary'
        db_table = 'beneficiary'



# Generated by Django 2.2.6 on 2019-12-18 12:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sender', '0001_initial'),
        ('beneficiary', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('10000', 'CREATED'), ('20000', 'CONFIRMED'), ('20110', 'CONFIRMED-UNDER-REVIEW-SLS'), ('20150', 'CONFIRMED-WAITING-FOR-PICKUP'), ('30000', 'REJECTED'), ('30110', 'REJECTED-SLS-SENDER'), ('30120', 'REJECTED-SLS-BENEFICIARY'), ('30200', 'REJECTED-INVALID-BENEFICIARY'), ('30201', 'REJECTED-BARRED-BENEFICIARY'), ('30210', 'REJECTED-INVALID-BENEFICIARY-DETAILS'), ('30305', 'REJECTED-LIMITATIONS-ON-TRANSACTION-VALUE'), ('30310', 'REJECTED-LIMITATIONS-ON-SENDER-VALUE'), ('30320', 'REJECTED-LIMITATIONS-ON-BENEFICIARY-VALUE'), ('30330', 'REJECTED-LIMITATIONS-ON-ACCOUNT-VALUE'), ('30350', 'REJECTED-LIMITATIONS-ON-SENDER-QUANTITY'), ('30360', 'REJECTED-LIMITATIONS-ON-BENEFICIARY-QUANTITY'), ('30370', 'REJECTED-LIMITATIONS-ON-ACCOUNT-QUANTITY'), ('30400', 'REJECTED-PAYER-CURRENTLY-UNAVAILABLE'), ('30500', 'REJECTED-INSUFFICIENT-BALANCE'), ('40000', 'CANCELLED'), ('50000', 'SUBMITTED'), ('60000', 'AVAILABLE'), ('70000', 'COMPLETED'), ('80000', 'REVERSED'), ('90000', 'DECLINED'), ('90110', 'DECLINED-SLS-SENDER'), ('90120', 'DECLINED-SLS-BENEFICIARY'), ('90200', 'DECLINED-INVALID-BENEFICIARY'), ('90201', 'DECLINED-BARRED-BENEFICIARY'), ('90202', 'DECLINED-UNSUPPORTED-BENEFICIARY'), ('90210', 'DECLINED-INVALID-BENEFICIARY-DETAILS'), ('90305', 'DECLINED-LIMITATIONS-ON-TRANSACTION-VALUE'), ('90310', 'DECLINED-LIMITATIONS-ON-SENDER-VALUE'), ('90320', 'DECLINED-LIMITATIONS-ON-BENEFICIARY-VALUE'), ('90330', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE'), ('90331', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-DAILY'), ('90332', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-WEEKLY'), ('90333', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-MONTHLY'), ('90334', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-YEARLY'), ('90350', 'DECLINED-LIMITATIONS-ON-SENDER-QUANTITY'), ('90360', 'DECLINED-LIMITATIONS-ON-BENEFICIARY-QUANTITY'), ('90370', 'DECLINED-LIMITATIONS-ON-ACCOUNT-QUANTITY'), ('90400', 'DECLINED-PAYER-CURRENTLY-UNAVAILABLE')], default='10000', max_length=50, verbose_name='Transaction status code')),
                ('status_message', models.CharField(choices=[('10000', 'CREATED'), ('20000', 'CONFIRMED'), ('20110', 'CONFIRMED-UNDER-REVIEW-SLS'), ('20150', 'CONFIRMED-WAITING-FOR-PICKUP'), ('30000', 'REJECTED'), ('30110', 'REJECTED-SLS-SENDER'), ('30120', 'REJECTED-SLS-BENEFICIARY'), ('30200', 'REJECTED-INVALID-BENEFICIARY'), ('30201', 'REJECTED-BARRED-BENEFICIARY'), ('30210', 'REJECTED-INVALID-BENEFICIARY-DETAILS'), ('30305', 'REJECTED-LIMITATIONS-ON-TRANSACTION-VALUE'), ('30310', 'REJECTED-LIMITATIONS-ON-SENDER-VALUE'), ('30320', 'REJECTED-LIMITATIONS-ON-BENEFICIARY-VALUE'), ('30330', 'REJECTED-LIMITATIONS-ON-ACCOUNT-VALUE'), ('30350', 'REJECTED-LIMITATIONS-ON-SENDER-QUANTITY'), ('30360', 'REJECTED-LIMITATIONS-ON-BENEFICIARY-QUANTITY'), ('30370', 'REJECTED-LIMITATIONS-ON-ACCOUNT-QUANTITY'), ('30400', 'REJECTED-PAYER-CURRENTLY-UNAVAILABLE'), ('30500', 'REJECTED-INSUFFICIENT-BALANCE'), ('40000', 'CANCELLED'), ('50000', 'SUBMITTED'), ('60000', 'AVAILABLE'), ('70000', 'COMPLETED'), ('80000', 'REVERSED'), ('90000', 'DECLINED'), ('90110', 'DECLINED-SLS-SENDER'), ('90120', 'DECLINED-SLS-BENEFICIARY'), ('90200', 'DECLINED-INVALID-BENEFICIARY'), ('90201', 'DECLINED-BARRED-BENEFICIARY'), ('90202', 'DECLINED-UNSUPPORTED-BENEFICIARY'), ('90210', 'DECLINED-INVALID-BENEFICIARY-DETAILS'), ('90305', 'DECLINED-LIMITATIONS-ON-TRANSACTION-VALUE'), ('90310', 'DECLINED-LIMITATIONS-ON-SENDER-VALUE'), ('90320', 'DECLINED-LIMITATIONS-ON-BENEFICIARY-VALUE'), ('90330', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE'), ('90331', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-DAILY'), ('90332', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-WEEKLY'), ('90333', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-MONTHLY'), ('90334', 'DECLINED-LIMITATIONS-ON-ACCOUNT-VALUE-YEARLY'), ('90350', 'DECLINED-LIMITATIONS-ON-SENDER-QUANTITY'), ('90360', 'DECLINED-LIMITATIONS-ON-BENEFICIARY-QUANTITY'), ('90370', 'DECLINED-LIMITATIONS-ON-ACCOUNT-QUANTITY'), ('90400', 'DECLINED-PAYER-CURRENTLY-UNAVAILABLE')], default='10000', max_length=50, verbose_name='Transaction status description')),
                ('status_class', models.CharField(choices=[('1', 'CREATED'), ('2', 'CONFIRMED'), ('3', 'REJECTED'), ('4', 'CANCELLED'), ('5', 'SUBMITTED'), ('6', 'AVAILABLE'), ('7', 'COMPLETED'), ('8', 'REVERSED'), ('9', 'DECLINED')], default='1', max_length=50, verbose_name='Transaction status class')),
                ('status_class_message', models.CharField(choices=[('1', 'CREATED'), ('2', 'CONFIRMED'), ('3', 'REJECTED'), ('4', 'CANCELLED'), ('5', 'SUBMITTED'), ('6', 'AVAILABLE'), ('7', 'COMPLETED'), ('8', 'REVERSED'), ('9', 'DECLINED')], default='1', max_length=50)),
                ('external_id', models.CharField(max_length=50, verbose_name='External ID')),
                ('external_code', models.CharField(max_length=50, verbose_name='External reference code')),
                ('payer_transaction_reference', models.CharField(blank=True, max_length=50, verbose_name='Payer transaction reference')),
                ('payer_transaction_code', models.CharField(blank=True, max_length=50, verbose_name='Payer transaction code')),
                ('creation_date', models.DateTimeField(max_length=50, verbose_name='Creation date in HTTP format')),
                ('expiration_date', models.DateTimeField(max_length=50, verbose_name='Expiration date in HTTP format')),
                ('callback_url', models.CharField(max_length=50, verbose_name='Callback URL')),
                ('wholesale_fx_rate', models.DecimalField(decimal_places=15, max_digits=15, verbose_name='Wholesale FX rate')),
                ('retail_rate', models.DecimalField(decimal_places=15, max_digits=15, verbose_name='Retail rate')),
                ('retail_fee', models.DecimalField(decimal_places=15, max_digits=15, verbose_name='Retail fee')),
                ('retail_fee_currency', models.CharField(max_length=50, verbose_name='Retail fee currency in ISO 4217 format')),
                ('purpose_of_remittance', models.CharField(max_length=50, verbose_name='Purpose of remittance')),
                ('additional_information_1', models.CharField(blank=True, max_length=50, verbose_name='Additional information')),
                ('additional_information_2', models.CharField(blank=True, max_length=50, verbose_name='Additional information')),
                ('additional_information_3', models.CharField(blank=True, max_length=50, verbose_name='Additional information')),
                ('beneficiary', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='beneficiary', to='beneficiary.Beneficiary', verbose_name='Beneficiary information')),
                ('sender', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='sender', to='sender.Sender', verbose_name='Sender information')),
            ],
            options={
                'verbose_name': 'transaction',
                'db_table': 'transaction',
                'index_together': {('sender', 'beneficiary')},
            },
        ),
    ]



# Generated by Django 2.2.6 on 2019-12-18 12:10

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Beneficiary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('lastname', models.CharField(max_length=50, verbose_name='Last Name')),
                ('lastname2', models.CharField(blank=True, max_length=50, verbose_name='Additional last name(s)')),
                ('middlename', models.CharField(blank=True, max_length=50, verbose_name='Middle Name')),
                ('firstname', models.CharField(max_length=50, verbose_name='First Name')),
                ('nativename', models.CharField(blank=True, max_length=50, verbose_name='Full name in native characters')),
                ('nationality_country_iso_code', models.CharField(max_length=10, verbose_name='National Country ISO Code')),
                ('code', models.CharField(blank=True, max_length=10, verbose_name='code')),
                ('data_of_birth', models.DateField(verbose_name='Date Of Birth')),
                ('country_of_birth_iso_code', models.CharField(max_length=10, verbose_name='Country Of Birth ISO Code')),
                ('gender', models.CharField(choices=[('MALE', 'Male'), ('FEMALE', 'Female')], default='MALE', max_length=10, verbose_name='gender')),
                ('address', models.CharField(max_length=50, verbose_name='Address')),
                ('postal_code', models.CharField(max_length=50, verbose_name='Postal Code')),
                ('city', models.CharField(max_length=50, verbose_name='City')),
                ('country_iso_code', models.CharField(max_length=50, verbose_name='Country ISO Code')),
                ('msisdn', models.CharField(max_length=50, verbose_name='MSISDN in international format')),
                ('email', models.EmailField(max_length=50, verbose_name='Email Address')),
                ('id_type', models.CharField(choices=[('PASSPORT', 'Passport'), ('NATIONAL_ID', 'National Identification Card'), ('DRIVING_LICENSE', 'Driving License'), ('SOCIAL_SECURITY', 'Social Security Card/Number'), ('TAX_ID', 'Tax Payer Identification Card/Number'), ('SENIOR_CITIZEN_ID', 'Senior Citizen Identification Card'), ('BIRTH_CERTIFICATE', 'Birth Certificate'), ('VILLAGE_ELDER_ID', 'Village Elder Identification Card'), ('RESIDENT_CARD', 'Permanent Residency Identification Card'), ('ALIEN_REGISTRATION', 'Alien Registration Certificate/Card'), ('PAN_CARD', 'PAN Card'), ('VOTERS_ID', 'Voter’s Identification Card'), ('HEALTH_CARD', 'Health Insurance Card/Number'), ('EMPLOYER_ID', 'Employer Identification Card'), ('OTHER', 'Others not listed')], default='', max_length=50, verbose_name='Presented identification type')),
                ('id_country_iso_code', models.CharField(max_length=50, verbose_name='ID country in ISO 3166-1 alpha-3 format')),
                ('id_number', models.CharField(max_length=50, verbose_name='Presented identification number')),
                ('id_delivery_date', models.DateField(blank=True, max_length=50, verbose_name='ID delivery date in ISO 8601 format')),
                ('id_expiration_date', models.DateField(blank=True, max_length=50, null=True, verbose_name='ID expiration date in ISO 8601 format')),
                ('occupation', models.CharField(max_length=50, verbose_name='Occupation')),
                ('bank_accout_holder_name', models.CharField(blank=True, max_length=50, verbose_name='Bank account holder name')),
                ('province_state', models.CharField(blank=True, max_length=50, verbose_name='Address province/state')),
            ],
            options={
                'verbose_name': 'beneficiary',
                'db_table': 'beneficiary',
            },
        ),
    ]



from rest_framework import serializers

from .models import Quotation, Destination, Source


class SourceSerializer(serializers.ModelSerializer):

    class Meta:
        model = Source
        fields = '__all__'


class DestinationSerializer(serializers.ModelSerializer):

    class Meta:
        model = Destination
        fields = '__all__'


class QuotationSerializer(serializers.ModelSerializer):

    class Meta:
        model = Quotation
        fields = '__all__'



"""
WSGI config for thunes project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/howto/deployment/wsgi/
"""

import os

from django.core.wsgi import get_wsgi_application

# use the settings for the develop
profile = os.environ.get('PROJECT_PROFILE', 'develop')
# use the settings for the product
# profile = os.environ.get('PROJECT_PROFILE', 'product')
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'thunes.settings.%s' % profile)

application = get_wsgi_application()



from django.contrib import admin

from .models import Transaction

# Register your models here.
@admin.register(Transaction)
class TransactionAdmin(admin.ModelAdmin):
    list_display = ['status', 'status_message', 'status_class', 'status_class_message', 'external_id',
                    'external_code', 'payer_transaction_reference', 'payer_transaction_code', 'creation_date',
                    'expiration_date', 'callback_url', 'wholesale_fx_rate', 'retail_rate',
                    'retail_fee', 'retail_fee_currency', 'purpose_of_remittance']




# Generated by Django 2.2.6 on 2019-12-21 08:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('beneficiary', '0002_auto_20191221_1611'),
    ]

    operations = [
        migrations.AlterField(
            model_name='beneficiary',
            name='id_delivery_date',
            field=models.DateField(blank=True, max_length=50, null=True, verbose_name='ID delivery date in ISO 8601 format'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-12-23 08:17

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('beneficiary', '0003_auto_20191221_1619'),
        ('sender', '0002_auto_20191221_1611'),
        ('transaction', '0006_auto_20191221_1659'),
    ]

    operations = [
        migrations.CreateModel(
            name='TransactionSenderBeneficiary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('beneficiary', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='beneficiary.Beneficiary', verbose_name='beneficiary entity in this transaction')),
                ('sender', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='sender.Sender', verbose_name='sender entity in this transaction')),
            ],
        ),
    ]



# Generated by Django 2.2.6 on 2019-12-21 05:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('transaction', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transaction',
            name='beneficiary',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='beneficiaries', to='beneficiary.Beneficiary', verbose_name='Beneficiary information'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='sender',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='senders', to='sender.Sender', verbose_name='Sender information'),
        ),
    ]



from setuptools import setup, find_packages

setup(
    name='thunes',
    version='0.23',
    description='Thunes Technical Test on Django',
    author='zhkai',
    author_email='jackson.zhang1@hotmail.com',
    url='https://github.com/Godcrying123/',
    license='MIT',
    packages=find_packages('thunes'),
    package_dir={'': 'thunes'},
    # package_data={'': [
    #     'shopsite/shopsite/*',
    # ]},
    include_package_data=True,
    install_requires=[
        # 'django==2.2.6',
        # 'Pillow==6.2.0',
        # 'WeasyPrint==0.42.3',
        # 'braintree==3.57.1',
        # 'celery==4.3.0',
        # 'coreapi==2.3.3',
        # 'django-parler==2.0',
        # 'django-rosetta==0.8.1',
        # 'djangorestframework==3.10.3',
        # 'eventlet==0.25.1',
        # 'flower==0.9.2',
        # 'Jinja2==2.10.3',
        # 'Pygments==2.4.2',
        # 'amqp==2.5.1',
        # 'django-redis==4.10.0',
        # 'redis==3.3.10',
        # 'hiredis==1.0.0',
        # # 'mysqlclient==1.4.4',
        # 'pymysql==0.9.3',
    ],
    extras_require={
      'ipython': ['ipython==7.8.0']
    },
    scripts=[
        'thunes/manage.py',
    ],
    entry_points={
        'console_scripts': [
            'thunes_manage = manage:main',
        ]
    },
    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.7',
    ],
)



# Generated by Django 2.2.6 on 2019-12-21 08:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('transaction', '0002_auto_20191221_1340'),
    ]

    operations = [
        migrations.CreateModel(
            name='CreditPartyIdentity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('msisdn', models.CharField(max_length=50, verbose_name='MSISDN in international format')),
                ('bank_account_number', models.CharField(max_length=50, verbose_name='Bank account number')),
                ('swift_bic_code', models.CharField(max_length=50, verbose_name='SWIFT-BIC code')),
            ],
            options={
                'db_table': 'Credit Party Identity',
            },
        ),
        migrations.AlterField(
            model_name='transaction',
            name='creation_date',
            field=models.DateTimeField(auto_created=True, max_length=50, verbose_name='Creation date in HTTP format'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='expiration_date',
            field=models.DateTimeField(blank=True, max_length=50, null=True, verbose_name='Expiration date in HTTP format'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='wholesale_fx_rate',
            field=models.DecimalField(blank=True, decimal_places=15, max_digits=15, verbose_name='Wholesale FX rate'),
        ),
    ]



"""
Django settings for thunes project.

Generated by 'django-admin startproject' using Django 2.2.6.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os
import configparser

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '_@_ubd406-94p3(1-t*aqnae8-xgjlpb!noefx5l60()uc0gj$'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'sender.apps.SenderConfig',
    'beneficiary.apps.BeneficiaryConfig',
    'transaction.apps.TransactionConfig',
    'quotation.apps.QuotationConfig',
    'rest_framework',
    'rest_framework_swagger',
    'corsheaders',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    # 'django.middleware.csrf.CsrfResponseMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'thunes.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'thunes.wsgi.application'


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = "/var/www/example.com/static/"

# Rest Framework Settings for permissions and pagination
REST_FRAMEWORK = {
    'DEFAULT_SCHEMA_CLASS': 'rest_framework.schemas.coreapi.AutoSchema'
}


# Configure the path for external API Secret and API Key
dir1 = os.path.abspath(os.path.dirname("base.py"))
dirAPISecret = os.path.dirname(dir1)

conf = configparser.ConfigParser()
conf.read(dirAPISecret + '/APISecret.ini', encoding='utf-8')

# Thunes API calling parameters
API_BASE_URL = str(conf.get('Global', 'API_BASE_URL'))
API_KEY = str(conf.get('Global', 'API_KEY'))
API_SECRET = str(conf.get('Global', 'API_SECRET'))



from django.contrib import admin

from .models import Beneficiary
# Register your models here.


@admin.register(Beneficiary)
class BeneficiaryAdmin(admin.ModelAdmin):
    list_display = ['lastname', 'firstname', 'nationality_country_iso_code', 'date_of_birth', 'gender',
                    'country_of_birth_iso_code', 'address', 'postal_code', 'city', 'country_iso_code',
                    'msisdn', 'email', 'id_type', 'id_country_iso_code', 'id_number', 'id_delivery_date',
                    'occupation']
    list_editable = ['date_of_birth', 'gender', 'id_type', 'id_delivery_date']

